import xbmc
from math import floor

from resources.lib.const import STORAGE, PLAYER_SERVICE_EVENT, SERVICE, SETTINGS, STORAGE
from resources.lib.kodilogging import service_logger
from resources.lib.services import Service
from resources.lib.storage.settings import settings
from resources.lib.storage.sqlite import SQLiteStorage
from resources.lib.storage.storage import storage
from resources.lib.api.trakt_api import trakt

class PlayerService(Service):
    SERVICE_NAME = SERVICE.PLAYER_SERVICE

    def __init__(self, api):
        super(PlayerService, self).__init__()
        self._api = api
        self._event_callbacks = {
            PLAYER_SERVICE_EVENT.PLAYBACK_STARTED: self.media_played,
            PLAYER_SERVICE_EVENT.PLAYBACK_STOPPED: self.media_stopped,
            PLAYER_SERVICE_EVENT.TRAKT_AUTHENTICATED: self.trakt_authenticated
        }
        self.db = SQLiteStorage.WatchHistory()
        self.trakt_id = None
        self.current_time = 0
        self.total_time = 0

    def media_played(self, media_id, trakt_id=None):
        service_logger.debug('Sending API request to increment play count')
        self._api.media_played(media_id)
        if settings.as_bool(SETTINGS.PLAYED_ITEMS_HISTORY):
            self.db.add(media_id)

        service_logger.debug('PlayerService.media_played %r' % trakt_id)
        if trakt.is_authenticated() and trakt_id:
            self.trakt_id = trakt_id

    def periodicalCheck(self):
        if not xbmc.Player().isPlayingVideo():
            return
        try:
            self.current_time = xbmc.Player().getTime()
            self.total_time = xbmc.Player().getTotalTime()
        except Exception:
            pass

    def scrobble_start(self):
        service_logger.debug('PlayerService.scrobble_start')
        if self.trakt_id:
            trakt.scrobble_start(self.trakt_id, self.watchedPercent())

    def scrobble_pause(self):
        service_logger.debug('PlayerService.scrobble_pause')
        if self.trakt_id:
            trakt.scrobble_pause(self.trakt_id, self.watchedPercent())

    def scrobble_stop(self):
        service_logger.debug('PlayerService.scrobble_stop')
        if self.trakt_id:
            progress = self.watchedPercent()
            trakt.scrobble_stop(self.trakt_id, progress)
            self.trakt_id = None
            self.current_time = 0
            self.watched_time = 0

    def trakt_authenticated(self):
        service_logger.debug('PlayerService.trakt_authenticated')
        trakt.authenticate()

    def watchedPercent(self):
        self.periodicalCheck()
        try:
            result = self.current_time/self.total_time*100
        except ZeroDivisionError:
            result = 0
        # service_logger.debug('PlayerService.watchedPercent current: %d, total: %d, perecent: %d%%' % (self.current_time, self.total_time, result))
        return result

    @staticmethod
    def media_stopped():
        service_logger.debug('Playback stopped.')
